import { Body, Controller, Del, Get, Inject, Param, Post, Query } from '@midwayjs/decorator';
import { BaseController } from 'free-midway-component';
import { GoodsConfigService } from '../service/goodsConfig.service';
import {
  CreateDTO_goodsConfig,
  CreateVO_goodsConfig,
  FetchVO_goodsConfig,
  QueryDTO_goodsConfig,
  QueryVO_goodsConfig,
  UpdateDTO_goodsConfig,
  UpdateVO_goodsConfig,
} from '../interface/goodsConfig.interface';
import { ApiQuery, ApiResponse, ApiTags } from '@midwayjs/swagger';
import { Validate } from '@midwayjs/validate';
@ApiTags(['商品设置'])
@Controller('/goodsConfig')
export class GoodsConfigController extends BaseController {
  @Inject()
  service: GoodsConfigService;

  @Post('/')
  @Validate()
  @ApiResponse({ type: CreateVO_goodsConfig })
  async create(@Body() data: CreateDTO_goodsConfig) {
    const result = await this.service.create(data);
    return result;
  }
  @Get('/')
  @ApiQuery({
    name: 'query',
  })
  @ApiResponse({ type: QueryVO_goodsConfig })
  async query(@Query() filter: QueryDTO_goodsConfig, @Query('skip') skip: number, @Query('limit') limit: number) {
    const data = await this.service.query(filter, { skip, limit, sort: { 'meta.createdAt': 1 } });
    const total = await this.service.count(filter);
    return { data, total };
  }

  @Get('/:id')
  @ApiResponse({ type: FetchVO_goodsConfig })
  async fetch(@Param('id') id: string) {
    const data = await this.service.fetch(id, { populate: false });
    const result = new FetchVO_goodsConfig(data);
    return result;
  }

  @Post('/:id')
  @Validate()
  @ApiResponse({ type: UpdateVO_goodsConfig })
  async update(@Param('id') id: string, @Body() body: UpdateDTO_goodsConfig) {
    const result = await this.service.updateOne(id, body);
    return result;
  }

  @Del('/:id')
  @Validate()
  async delete(@Param('id') id: string) {
    await this.service.delete(id);
    return 'ok';
  }
  async createMany(...args: any[]) {
    throw new Error('Method not implemented.');
  }

  async updateMany(...args: any[]) {
    throw new Error('Method not implemented.');
  }

  async deleteMany(...args: any[]) {
    throw new Error('Method not implemented.');
  }
}
