import { Rule, RuleType } from '@midwayjs/validate';
import { ApiProperty } from '@midwayjs/swagger';
import _ = require('lodash');
import { FrameworkErrorEnum, SearchBase, ServiceError } from 'free-midway-component';
import { getDataFromTarget } from '../util/util';
import * as computedUtil from '../util/computed';
export class FetchVO_group {
  constructor(data: object) {
    for (const key of Object.keys(this)) {
      this[key] = _.get(data, key);
    }
  }
  @ApiProperty({ description: '数据id' })
  _id: string = undefined;
  @ApiProperty({ description: '商店id' })
  'shop': string = undefined;
  @ApiProperty({ description: '商品id' })
  'goods': string = undefined;
  @ApiProperty({ description: '团长' })
  'leader': string = undefined;
  @ApiProperty({ description: '人数限制' })
  'person_limit': number = undefined;
  @ApiProperty({ description: '团购设置' })
  'group_config': Array<any> = undefined;
  @ApiProperty({ description: '开始时间' })
  'start_time': string = undefined;
  @ApiProperty({ description: '结束时间' })
  'end_time': string = undefined;
  @ApiProperty({ description: '团状态' })
  'status': string = undefined;
}

export class QueryDTO_group extends SearchBase {
  constructor() {
    const like_prop = ['goods'];
    const props = ['shop', 'goods', 'leader', 'status'];
    const mapping = { goods: 'goods.name', leader: 'leader_id' };
    super({ like_prop, props, mapping });
  }
  @ApiProperty({ description: '商店id' })
  'shop': string = undefined;
  @ApiProperty({ description: '商品名称' })
  'goods': string = undefined;
  @ApiProperty({ description: '团长' })
  'leader': string = undefined;
  @ApiProperty({ description: '状态' })
  'status': string = undefined;
}

export class QueryVO_group extends FetchVO_group {}

export class CreateDTO_group {
  @ApiProperty({ description: '商店id' })
  @Rule(RuleType['string']().required().error(new ServiceError('缺少商店id', FrameworkErrorEnum.NEED_BODY)))
  'shop': string = undefined;
  @ApiProperty({ description: '商品id' })
  @Rule(RuleType['string']().required().error(new ServiceError('缺少商品id', FrameworkErrorEnum.NEED_BODY)))
  'goods': string = undefined;
  @ApiProperty({ description: '团长' })
  @Rule(RuleType['string']().empty())
  'leader': string = undefined;
  @ApiProperty({ description: '人数限制' })
  @Rule(RuleType['number']().required().error(new ServiceError('缺少人数限制', FrameworkErrorEnum.NEED_BODY)))
  'person_limit': number = undefined;
  @ApiProperty({ description: '团购设置' })
  @Rule(RuleType['array']().empty())
  'group_config': Array<any> = undefined;
  @ApiProperty({ description: '开始时间' })
  @Rule(RuleType['string']().empty())
  'start_time': string = undefined;
  @ApiProperty({ description: '结束时间' })
  @Rule(RuleType['string']().empty())
  'end_time': string = undefined;
  @ApiProperty({ description: '团状态' })
  @Rule(RuleType['string']().empty())
  'status': string = undefined;
}

export class CreateVO_group extends FetchVO_group {}

export class UpdateDTO_group {
  @ApiProperty({ description: '数据id' })
  @Rule(RuleType['string']().empty())
  '_id'?: string = undefined;
  @ApiProperty({ description: '商店id' })
  @Rule(RuleType['string']().empty())
  'shop'?: string = undefined;
  @ApiProperty({ description: '商品id' })
  @Rule(RuleType['string']().empty())
  'goods'?: string = undefined;
  @ApiProperty({ description: '团长' })
  @Rule(RuleType['string']().empty())
  'leader'?: string = undefined;
  @ApiProperty({ description: '人数限制' })
  @Rule(RuleType['number']().empty())
  'person_limit'?: number = undefined;
  @ApiProperty({ description: '团购设置' })
  @Rule(RuleType['array']().empty())
  'group_config'?: Array<any> = undefined;
  @ApiProperty({ description: '开始时间' })
  @Rule(RuleType['string']().empty())
  'start_time'?: string = undefined;
  @ApiProperty({ description: '结束时间' })
  @Rule(RuleType['string']().empty())
  'end_time'?: string = undefined;
  @ApiProperty({ description: '团状态' })
  @Rule(RuleType['string']().empty())
  'status'?: string = undefined;
}

export class UpdateVO_group extends FetchVO_group {}
export class UserListView {
  constructor(data) {
    const dl = ['_id', 'can_group', 'end_time', 'start_time', 'person_limit', 'status'];
    for (const i of dl) this[i] = _.get(data, i);
    this.goods = getDataFromTarget(data, 'goods', ['_id', 'name', 'file']);
    this.shop = getDataFromTarget(data, 'shop', ['_id', 'name']);
    const gc = _.minBy(data.group_config, 'price');
    const group_price = _.get(gc, 'price', 0);
    this.group_price = computedUtil.toNumber(group_price);
    const sell_price = _.get(gc, 'spec.sell_money', 0);
    this.sell_price = computedUtil.toNumber(sell_price);
  }
  _id: string;
  end_time: string;
  start_time: string;
  person_limit: number;
  goods: {
    _id: string;
    name: string;
    file: Array<any>;
  };
  shop: {
    _id: string;
    name: string;
  };
  status: string;
  can_group: boolean;
  group_price: number;
  sell_price: number;
}

export class AqView extends UserListView {
  constructor(data) {
    super(data);
    this.leader_name = _.get(data, 'leader.name');
  }
  leader_name: string;
}
