import { Inject, App, Config, Provide, ScopeEnum, Scope, Init, Autoload } from '@midwayjs/decorator';
import { Context, Application } from '@midwayjs/rabbitmq';
import * as amqp from 'amqp-connection-manager';
import { ConsumeMessage } from 'amqplib';
import _ = require('lodash');
import { PayController } from '../../controller/pay.controller';
import { GroupService } from '../group.service';
import { GroupController } from '../../controller/group.controller';
@Autoload()
@Provide()
@Scope(ScopeEnum.Request, { allowDowngrade: true })
export class MqConsumer {
  @App()
  app: Application;

  @Inject()
  ctx: Context;

  @Config('rabbitmq.url')
  mqUrl: object;

  @Config('mqConfig.dead')
  mqDeadConfig: object;
  @Inject()
  payController: PayController;

  @Inject()
  groupService: GroupService;

  @Inject()
  groupController: GroupController;

  /**mq连接 */
  conn;
  /**订单频道 */
  order_ch;
  @Init()
  async init() {
    this.conn = await amqp.connect(this.mqUrl);
    this.order_ch = this.conn.createChannel({
      json: true,
      setup: ch => {
        return Promise.all([
          ch.assertExchange(_.get(this.mqDeadConfig, 'ex'), 'direct', { durable: true }),
          ch.assertQueue(_.get(this.mqDeadConfig, 'q'), { durable: true, exclusive: false }),
          ch.bindQueue(_.get(this.mqDeadConfig, 'd'), _.get(this.mqDeadConfig, 'ex'), _.get(this.mqDeadConfig, 'rk')),
          ch.consume(_.get(this.mqDeadConfig, 'q'), msg => this.dealOrder.call(this, msg), { noAck: true }),
        ]);
      },
    });
  }

  /**
   * 处理死信信息,订单过期
   * @param data mq数据
   */
  async dealOrder(data: ConsumeMessage) {
    const str = data.content.toString();
    const obj = JSON.parse(str);
    const type = _.get(obj, 'type');
    const params = _.omit(obj, ['type']);
    if (type === 'order') await this.payController.cancel(params);
    else if (type === 'group') {
      const id = _.get(params, 'id');
      // 先查询该团是否满足开团的条件了.满足的可以直接走
      const result = await this.groupService.autoCheck(params);
      // 没有异常,结果不是布尔值,说明不需要处理/参数错误,不予处理
      if (!_.isBoolean(result)) return;
      else await this.groupService.sendSystemMsg(id);
    }
  }
}
