import { App, Provide, Scope, ScopeEnum, Init, Autoload, Destroy, Config } from '@midwayjs/decorator';
import * as amqp from 'amqp-connection-manager';
import { Application } from '@midwayjs/koa';
import _ = require('lodash');
import * as computedUtil from '../../util/computed';
@Autoload()
@Provide()
@Scope(ScopeEnum.Singleton)
export class MqSender {
  @App()
  app: Application;

  private connection: amqp.AmqpConnectionManager;

  private channelWrapper;

  @Config('rabbitmq.url')
  mqUrl: object;

  @Config('mqConfig.normal')
  mqConfig: object;
  @Config('mqConfig.dead')
  mqDeadConfig: object;

  @Config('mqConfig.timeout')
  timeout: number;

  @Init()
  async connect() {
    // 创建连接，你可以把配置放在 Config 中，然后注入进来
    this.connection = await amqp.connect(this.mqUrl);
    // 创建 channel
    this.channelWrapper = this.connection.createChannel({
      json: true,
      setup: channel => {
        return Promise.all([
          // 绑定队列
          channel.assertExchange(_.get(this.mqConfig, 'ex'), 'direct', { durable: true }),
          channel.assertQueue(_.get(this.mqConfig, 'q'), {
            durable: true,
            exclusive: false,
            deadLetterExchange: _.get(this.mqDeadConfig, 'ex'),
            deadLetterRoutingKey: _.get(this.mqDeadConfig, 'rk'),
          }),
          channel.bindQueue(_.get(this.mqConfig, 'q'), _.get(this.mqConfig, 'ex')),
        ]);
      },
    });
  }

  // 订单死信消息
  async orderMsg(data: object) {
    const q = _.get(this.mqConfig, 'q');
    const min = this.timeout;
    // 转换成毫秒
    const ms = computedUtil.multiply(min, 60000);
    return this.channelWrapper.sendToQueue(q, { ...data, type: 'order' }, { expiration: ms });
  }

  /**团死信消息 */
  async groupMsg(data: object, ms: number) {
    const q = _.get(this.mqConfig, 'q');
    return this.channelWrapper.sendToQueue(q, { ...data, type: 'group' }, { expiration: ms });
  }

  @Destroy()
  async close() {
    await this.channelWrapper.close();
    await this.connection.close();
  }
}
