import { Inject, Provide } from '@midwayjs/decorator';
import { HttpServiceFactory } from '@midwayjs/axios';
import { GoodsConfigService } from './goodsConfig.service';
import { Context } from '@midwayjs/koa';
import _ = require('lodash');
import { GroupService } from './group.service';
import { FrameworkErrorEnum, ServiceError } from 'free-midway-component';
import moment = require('moment');
import { InjectEntityModel } from '@midwayjs/typegoose';
import { ReturnModelType } from '@typegoose/typegoose';
import { GoodsSpec } from '../entity/base/goodsSpec';
@Provide()
export class ViewService {
  key = 'base';
  @Inject()
  ctx: Context;
  @Inject()
  httpServiceFactory: HttpServiceFactory;
  @Inject()
  goodsConfigService: GoodsConfigService;
  @Inject()
  groupService: GroupService;
  @InjectEntityModel(GoodsSpec)
  goodsSpecModel: ReturnModelType<typeof GoodsSpec>;

  /**
   * 整理视图所需数据
   * @param id 商品id
   * @param group 团id
   */
  async setGoodsDetailViewData({ id, group }) {
    const is_leader = _.get(this.ctx, 'user.is_leader', '1');
    // 把数据请求来,拿到这边再改
    const axios = this.httpServiceFactory.get(this.key);
    const result = await axios.post('/viewGoods/goodsDetail', { id });
    // 正常查看商品详情的数据
    const { data } = result;
    if (!data) throw new ServiceError('未找到商品详情', FrameworkErrorEnum.NOT_FOUND_DATA);
    // 需要处理下;1.不需要act
    delete data.act;
    // 2.检查团
    // const { result: gr, msg } = await this.groupService.checkGroup(group);
    // if (!gr) throw new ServiceError(msg, FrameworkErrorEnum.SERVICE_FAULT);
    const groupData = await this.groupService.fetch(group, { populate: false, lean: true });
    const group_config = _.get(groupData, 'group_config');
    // 整理规格
    const groupSpecs = [];
    for (const gc of group_config) {
      // 1.检查规格状态是否可以销售
      const { spec, leader_price, price: number_price, freight, buy_limit, limit_num } = gc;
      const num = await this.goodsSpecModel.count({ _id: spec._id, status: '0' });
      if (num <= 0) continue;
      // 限制购买默认继承了商品的规格设置.如果在商品设置处有设置过,则会使用商品设置处的设置,如果没有,就是用原设置
      const obj = _.pick(spec, ['_id', 'file', 'name', 'num', 'buy_limit', 'limit_num']);
      if (_.isObject(freight)) obj.freight = _.get(freight, '$numberDecimal');
      else obj.freight = freight;
      if (buy_limit) obj.buy_limit = buy_limit;
      if (limit_num) obj.limit_num = limit_num;
      obj.leader_price = leader_price;
      obj.number_price = number_price;
      // 不需要判断了,团购中,团长也按团员的价格走
      // if (is_leader === '0') obj.price = leader_price;
      // else
      obj.price = number_price;
      groupSpecs.push(obj);
    }
    data.specs = groupSpecs;
    return data;
  }
}
